@@ -4,6 +4,7 @@
 # Copyright: Ansible Project, 2020
 """Functions for working with the ansible-base package."""
 import ast
+import asyncio
 import os
 import re
 import tempfile
@@ -16,13 +17,16 @@ import sh
 from packaging.version import Version as PypiVer
 
 from . import app_context
-from .compat import best_get_loop
+from .compat import best_get_loop, create_task
+from .logging import log
 from .utils.http import retry_get
 
 if t.TYPE_CHECKING:
     import aiohttp.client
 
 
+mlog = log.fields(mod=__name__)
+
 #: URL to checkout ansible-base from.
 _ANSIBLE_BASE_URL = str(app_context.AppContext().ansible_base_url)
 #: URL to pypi.
@@ -51,22 +55,39 @@ class AnsibleBasePyPiClient:
         self.aio_session = aio_session
         self.pypi_server_url = pypi_server_url
 
-    @lru_cache(None)
-    async def get_info(self) -> t.Dict[str, t.Any]:
+    async def _get_json(self, query_url: str) -> t.Dict[str, t.Any]:
         """
-        Retrieve information about the ansible-base package from pypi.
-
-        :returns: The dict which represents the information about the ansible-base package returned
-            from pypi.  To examine the data structure, use::
-
-                curl https://pypi.org/pypi/ansible-base/json| python3 -m json.tool
+        JSON data from a url with retries and return the data as python data structures.
         """
-        # Retrieve the ansible-base package info from pypi
-        query_url = urljoin(self.pypi_server_url, 'pypi/ansible-base/json')
         async with retry_get(self.aio_session, query_url) as response:
             pkg_info = await response.json()
         return pkg_info
 
+    @lru_cache(None)
+    async def get_release_info(self) -> t.Dict[str, t.Any]:
+        """
+        Retrieve information about releases of the ansible-base/ansible-core package from pypi.
+
+        :returns: The dict which represents the release info keyed by version number.
+            To examine the data structure, use::
+
+                curl https://pypi.org/pypi/ansible-core/json| python3 -m json.tool
+
+        .. note:: Returns an aggregate of ansible-base and ansible-core releases.
+        """
+        # Retrieve the ansible-base and ansible-core package info from pypi
+        tasks = []
+        for package_name in ('ansible-core', 'ansible-base'):
+            query_url = urljoin(self.pypi_server_url, f'pypi/{package_name}/json')
+            tasks.append(create_task(self._get_json(query_url)))
+
+        # Note: gather maintains the order of results
+        results = await asyncio.gather(*tasks)
+        release_info = results[1]['releases']  # ansible-base information
+        release_info.update(results[0]['releases'])  # ansible-core information
+
+        return release_info
+
     async def get_versions(self) -> t.List[PypiVer]:
         """
         Get the versions of the ansible-base package on pypi.
@@ -74,9 +95,15 @@ class AnsibleBasePyPiClient:
         :returns: A list of :pypkg:obj:`packaging.versioning.Version`s
             for all the versions on pypi, including prereleases.
         """
-        pkg_info = await self.get_info()
-        versions = [PypiVer(r) for r in pkg_info['releases']]
+        flog = mlog.fields(func='AnsibleBasePyPiClient.get_versions')
+        flog.debug('Enter')
+
+        release_info = await self.get_release_info()
+        versions = [PypiVer(r) for r in release_info]
         versions.sort(reverse=True)
+        flog.fields(versions=versions).info('sorted list of ansible-core versions')
+
+        flog.debug('Leave')
         return versions
 
     async def get_latest_version(self) -> PypiVer:
@@ -97,16 +124,17 @@ class AnsibleBasePyPiClient:
         :arg download_dir: Directory to download the tarball to.
         :returns: The name of the downloaded tarball.
         """
-        pkg_info = await self.get_info()
+        package_name = get_ansible_core_package_name(ansible_base_version)
+        release_info = await self.get_release_info()
 
         pypi_url = tar_filename = ''
-        for release in pkg_info['releases'][ansible_base_version]:
-            if release['filename'].startswith(f'ansible-base-{ansible_base_version}.tar.'):
+        for release in release_info[ansible_base_version]:
+            if release['filename'].startswith(f'{package_name}-{ansible_base_version}.tar.'):
                 tar_filename = release['filename']
                 pypi_url = release['url']
                 break
         else:  # for-else: http://bit.ly/1ElPkyg
-            raise UnknownVersion(f'ansible-base {ansible_base_version} does not'
+            raise UnknownVersion(f'{package_name} {ansible_base_version} does not'
                                  ' exist on {self.pypi_server_url}')
 
         tar_filename = os.path.join(download_dir, tar_filename)
@@ -122,6 +150,23 @@ class AnsibleBasePyPiClient:
         return tar_filename
 
 
+def get_ansible_core_package_name(ansible_base_version: t.Union[str, PypiVer]) -> str:
+    """
+    Returns the name of the minimal ansible package.
+
+    :arg ansible_base_version: The version of the minimal ansible package to retrieve the
+        name for.
+    :returns: 'ansible-core' when the version is 2.11 or higher. Otherwise 'ansible-base'.
+    """
+    if not isinstance(ansible_base_version, PypiVer):
+        ansible_base_version = PypiVer(ansible_base_version)
+
+    if ansible_base_version.major <= 2 and ansible_base_version.minor <= 10:
+        return 'ansible-base'
+
+    return 'ansible-core'
+
+
 def _get_source_version(ansible_base_source: str) -> PypiVer:
     with open(os.path.join(ansible_base_source, 'lib', 'ansible', 'release.py')) as f:
         root = ast.parse(f.read())
@@ -275,6 +320,7 @@ async def get_ansible_base(aio_session: 'aiohttp.client.ClientSession',
     if ansible_base_version == '@devel':
         # is the source usable?
         if source_is_devel(ansible_base_source):
+            # source_is_devel() protects against this.  This assert is to inform the type checker
             assert ansible_base_source is not None
             source_location: str = ansible_base_source
 
