@@ -18,8 +18,10 @@ import asyncio_pool
 import sh
 from jinja2 import Template
 from packaging.version import Version as PypiVer
+from semantic_version import Version as SemVer, SimpleSpec as SemVerSpec
 
 from . import app_context
+from .ansible_base import get_ansible_core_package_name
 from .build_changelog import ReleaseNotes
 from .changelog import ChangelogData, get_changelog
 from .collections import install_separately, install_together
@@ -27,8 +29,6 @@ from .dependency_files import BuildFile, DependencyFileData, DepsFile
 from .galaxy import CollectionDownloader, GalaxyClient
 from .utils.get_pkg_data import get_antsibull_data
 
-from semantic_version import Version as SemVer
-
 
 #
 # Common code
@@ -37,7 +37,17 @@ from semantic_version import Version as SemVer
 
 async def get_collection_versions(deps: t.Mapping[str, str],
                                   galaxy_url: str,
+                                  pre: bool = False
                                   ) -> t.Dict[str, SemVer]:
+    """
+    Retrieve the latest version of each collection.
+
+    :arg deps: Mapping of collection name to a version specification.
+    :arg galaxy_url: The url for the galaxy server to use.
+    :kwarg pre: If True, allow prereleases (versions which have the form X.Y.Z-SOMETHING).
+        This is **not** for excluding 0.Y.Z versions.  The default is False.
+    :returns: Dict mapping collection name to latest version.
+    """
     requestors = {}
     async with aiohttp.ClientSession() as aio_session:
         lib_ctx = app_context.lib_ctx.get()
@@ -45,12 +55,12 @@ async def get_collection_versions(deps: t.Mapping[str, str],
             client = GalaxyClient(aio_session, galaxy_server=galaxy_url)
             for collection_name, version_spec in deps.items():
                 requestors[collection_name] = await pool.spawn(
-                    client.get_latest_matching_version(collection_name, version_spec))
+                    client.get_latest_matching_version(collection_name, version_spec, pre=pre))
 
             responses = await asyncio.gather(*requestors.values())
 
     # Note: Python dicts have a stable sort order and since we haven't modified the dict since we
-    # used requestors.values() to generate responses, requestors and responses therefor have
+    # used requestors.values() to generate responses, requestors and responses therefore have
     # a matching order.
     included_versions: t.Dict[str, SemVer] = {}
     for collection_name, version in zip(requestors, responses):
@@ -108,6 +118,16 @@ def write_manifest(package_dir: str,
         f.write('recursive-include ansible_collections/ **\n')
 
 
+def write_release_py(ansible_version: PypiVer, ansible_collections_dir: str) -> None:
+    release_filename = os.path.join(ansible_collections_dir, 'ansible_release.py')
+
+    release_tmpl = Template(get_antsibull_data('ansible-release_py.j2').decode('utf-8'))
+    release_contents = release_tmpl.render(version=ansible_version)
+
+    with open(release_filename, 'w') as f:
+        f.write(release_contents)
+
+
 def write_setup(ansible_version: PypiVer,
                 ansible_base_version: PypiVer,
                 collection_deps: str,
@@ -115,9 +135,11 @@ def write_setup(ansible_version: PypiVer,
     setup_filename = os.path.join(package_dir, 'setup.py')
 
     setup_tmpl = Template(get_antsibull_data('ansible-setup_py.j2').decode('utf-8'))
-    setup_contents = setup_tmpl.render(version=ansible_version,
-                                       ansible_base_version=ansible_base_version,
-                                       collection_deps=collection_deps)
+    setup_contents = setup_tmpl.render(
+        version=ansible_version,
+        ansible_core_package_name=get_ansible_core_package_name(ansible_base_version),
+        ansible_base_version=ansible_base_version,
+        collection_deps=collection_deps)
 
     with open(setup_filename, 'w') as f:
         f.write(setup_contents)
@@ -134,10 +156,13 @@ def write_python_build_files(ansible_version: PypiVer,
     write_setup(ansible_version, ansible_base_version, collection_deps, package_dir)
 
 
-def write_debian_directory(ansible_version: str, package_dir: str) -> None:
+def write_debian_directory(ansible_version: PypiVer,
+                           ansible_base_version: PypiVer,
+                           package_dir: str) -> None:
     debian_dir = os.path.join(package_dir, 'debian')
     os.mkdir(debian_dir, mode=0o700)
-    debian_files = ('changelog.j2', 'control', 'copyright', 'rules')
+    debian_files = ('changelog.j2', 'control.j2', 'copyright', 'rules')
+    ansible_core_package_name = get_ansible_core_package_name(ansible_base_version)
     for filename in debian_files:
         # Don't use os.path.join here, the get_data docs say it should be
         # slash-separated.
@@ -150,8 +175,9 @@ def write_debian_directory(ansible_version: str, package_dir: str) -> None:
             # and update 'data' to be the result.
             tmpl = Template(data)
             data = tmpl.render(
-                version=ansible_version,
-                date=datetime.datetime.utcnow().strftime("%a, %d %b %Y %T +0000"),
+                version=str(ansible_version),
+                date=datetime.datetime.utcnow().strftime('%a, %d %b %Y %T +0000'),
+                ansible_core_package_name=ansible_core_package_name,
             )
 
         with open(os.path.join(debian_dir, filename), 'w') as f:
@@ -202,30 +228,24 @@ def build_single_impl(dependency_data: DependencyFileData, add_release: bool = T
         asyncio.run(download_collections(included_versions, app_ctx.galaxy_url,
                                          download_dir, app_ctx.extra['collection_cache']))
 
-        if dependency_data is None:
-            dependency_data = DependencyFileData(
-                str(app_ctx.extra["ansible_version"]),
-                str(ansible_base_version),
-                {collection: str(version) for collection, version in included_versions.items()})
-
         # Get Ansible changelog, add new release
-        deps_dir = os.path.dirname(app_ctx.extra["build_file"])
-        ansible_changelog = ChangelogData.ansible(deps_dir)
+        ansible_changelog = ChangelogData.ansible(
+            app_ctx.extra['data_dir'], app_ctx.extra['dest_data_dir'])
         if add_release:
             date = datetime.date.today()
             ansible_changelog.add_ansible_release(
-                str(app_ctx.extra["ansible_version"]),
+                str(app_ctx.extra['ansible_version']),
                 date,
-                f"Release Date: {date}"
-                f"\n\n"
-                f"`Porting Guide <https://docs.ansible.com/ansible/devel/porting_guides.html>`_")
+                f'Release Date: {date}'
+                f'\n\n'
+                f'`Porting Guide <https://docs.ansible.com/ansible/devel/porting_guides.html>`_')
 
         # Get changelog and porting guide data
         changelog = get_changelog(
-            app_ctx.extra["ansible_version"],
-            deps_dir=deps_dir,
+            app_ctx.extra['ansible_version'],
+            deps_dir=app_ctx.extra['data_dir'],
             deps_data=[dependency_data],
-            collection_cache=app_ctx.extra["collection_cache"],
+            collection_cache=app_ctx.extra['collection_cache'],
             ansible_changelog=ansible_changelog)
 
         # Create package and collections directories
@@ -234,6 +254,9 @@ def build_single_impl(dependency_data: DependencyFileData, add_release: bool = T
         ansible_collections_dir = os.path.join(package_dir, 'ansible_collections')
         os.mkdir(ansible_collections_dir, mode=0o700)
 
+        # Write the ansible release info to the collections dir
+        write_release_py(app_ctx.extra['ansible_version'], ansible_collections_dir)
+
         # Install collections
         # TODO: PY3.8:
         # collections_to_install = [p for f in os.listdir(download_dir)
@@ -256,12 +279,13 @@ def build_single_impl(dependency_data: DependencyFileData, add_release: bool = T
         write_python_build_files(app_ctx.extra['ansible_version'], ansible_base_version, '',
                                  package_dir, release_notes, app_ctx.extra['debian'])
         if app_ctx.extra['debian']:
-            write_debian_directory(app_ctx.extra['ansible_version'], package_dir)
-        make_dist(package_dir, app_ctx.extra['dest_dir'])
+            write_debian_directory(app_ctx.extra['ansible_version'], ansible_base_version,
+                                   package_dir)
+        make_dist(package_dir, app_ctx.extra['sdist_dir'])
 
     # Write changelog and porting guide also to destination directory
-    release_notes.write_changelog_to(deps_dir)
-    release_notes.write_porting_guide_to(deps_dir)
+    release_notes.write_changelog_to(app_ctx.extra['dest_data_dir'])
+    release_notes.write_porting_guide_to(app_ctx.extra['dest_data_dir'])
 
     if add_release:
         ansible_changelog.changes.save()
@@ -270,25 +294,55 @@ def build_single_impl(dependency_data: DependencyFileData, add_release: bool = T
 def build_single_command() -> int:
     app_ctx = app_context.app_ctx.get()
 
-    build_file = BuildFile(app_ctx.extra['build_file'])
+    build_filename = os.path.join(app_ctx.extra['data_dir'], app_ctx.extra['build_file'])
+    build_file = BuildFile(build_filename)
     build_ansible_version, ansible_base_version, deps = build_file.parse()
     ansible_base_version = PypiVer(ansible_base_version)
+
+    # If we're building a feature frozen release (betas and rcs) then we need to
+    # change the upper version limit to not include new features.
+    if app_ctx.extra['feature_frozen']:
+        old_deps, deps = deps, {}
+        # For each collection that's listed...
+        for collection_name, spec in old_deps.items():
+            spec = SemVerSpec(spec)
+            new_clauses = []
+            min_version = None
+
+            # Look at each clause of the version specification
+            for clause in spec.clause.clauses:
+                if clause.operator in ('<', '<='):
+                    # Omit the upper bound as we're replacing it
+                    continue
+
+                if clause.operator == '>=':
+                    # Save the lower bound so we can write out a new compatible version
+                    min_version = clause.target
+
+                new_clauses.append(str(clause))
+
+            if min_version is None:
+                raise ValueError(f'No minimum version specified for {collection_name}: {spec}')
+
+            new_clauses.append(f'<{min_version.major}.{min_version.minor + 1}.0')
+            deps[collection_name] = ','.join(new_clauses)
+
     included_versions = asyncio.run(get_collection_versions(deps, app_ctx.galaxy_url))
 
     if not str(app_ctx.extra['ansible_version']).startswith(build_ansible_version):
-        print(f'{app_ctx.extra["build_file"]} is for version {build_ansible_version} but we need'
+        print(f'{build_filename} is for version {build_ansible_version} but we need'
               f' {app_ctx.extra["ansible_version"].major}'
               f'.{app_ctx.extra["ansible_version"].minor}')
         return 1
 
     dependency_data = DependencyFileData(
-        str(app_ctx.extra["ansible_version"]),
+        str(app_ctx.extra['ansible_version']),
         str(ansible_base_version),
         {collection: str(version) for collection, version in included_versions.items()})
 
     build_single_impl(dependency_data)
 
-    deps_filename = os.path.join(app_ctx.extra['dest_dir'], app_ctx.extra['deps_file'])
+    deps_filename = os.path.join(app_ctx.extra['dest_data_dir'], app_ctx.extra['deps_file'])
     deps_file = DepsFile(deps_filename)
     deps_file.write(
         dependency_data.ansible_version,
@@ -301,7 +355,7 @@ def build_single_command() -> int:
 def rebuild_single_command() -> int:
     app_ctx = app_context.app_ctx.get()
 
-    deps_filename = os.path.join(app_ctx.extra['dest_dir'], app_ctx.extra['deps_file'])
+    deps_filename = os.path.join(app_ctx.extra['data_dir'], app_ctx.extra['deps_file'])
     deps_file = DepsFile(deps_filename)
     dependency_data = deps_file.parse()
 
@@ -381,12 +435,15 @@ async def make_collection_dists(dest_dir: str, collection_dirs: t.List[str]) ->
 def build_multiple_command() -> int:
     app_ctx = app_context.app_ctx.get()
 
-    build_file = BuildFile(app_ctx.extra['build_file'])
+    build_filename = os.path.join(app_ctx.extra['data_dir'], app_ctx.extra['build_file'])
+    build_file = BuildFile(build_filename)
     build_ansible_version, ansible_base_version, deps = build_file.parse()
     ansible_base_version = PypiVer(ansible_base_version)
 
+    # TODO: implement --feature-frozen support
+
     if not str(app_ctx.extra['ansible_version']).startswith(build_ansible_version):
-        print(f'{app_ctx.extra["build_file"]} is for version {build_ansible_version} but we need'
+        print(f'{build_filename} is for version {build_ansible_version} but we need'
               f' {app_ctx.extra["ansible_version"].major}'
               f'.{app_ctx.extra["ansible_version"].minor}')
         return 1
@@ -409,7 +466,7 @@ def build_multiple_command() -> int:
                 collections_to_install.append(path)
 
         collection_dirs = asyncio.run(install_separately(collections_to_install, download_dir))
-        asyncio.run(make_collection_dists(app_ctx.extra['dest_dir'], collection_dirs))
+        asyncio.run(make_collection_dists(app_ctx.extra['sdist_dir'], collection_dirs))
 
         # Create the ansible package that deps on the collections we just wrote
         package_dir = os.path.join(tmp_dir, f'ansible-{app_ctx.extra["ansible_version"]}')
@@ -426,10 +483,10 @@ def build_multiple_command() -> int:
         write_python_build_files(app_ctx.extra['ansible_version'], ansible_base_version,
                                  collection_deps, package_dir)
 
-        make_dist(package_dir, app_ctx.extra['dest_dir'])
+        make_dist(package_dir, app_ctx.extra['sdist_dir'])
 
     # Write the deps file
-    deps_filename = os.path.join(app_ctx.extra['dest_dir'], app_ctx.extra['deps_file'])
+    deps_filename = os.path.join(app_ctx.extra['dest_data_dir'], app_ctx.extra['deps_file'])
     deps_file = DepsFile(deps_filename)
     deps_file.write(app_ctx.extra['ansible_version'], ansible_base_version, included_versions)
 
