@@ -29,6 +29,62 @@ from .changelog import Changelog, ChangelogData, ChangelogEntry, CollectionsMeta
 PluginDataT = t.List[t.Tuple[str, str, ChangelogGenerator, t.Optional[ChangelogGeneratorEntry]]]
 
 
+def optimize_release_entry(entry: ChangelogGeneratorEntry) -> ChangelogGeneratorEntry:
+    '''Remove duplicate entries from changelog entry.
+
+    This can happen if entries from versions from different changelogs are combined
+    which partially include each other's changes. This happens if for example the 1.1.0
+    changelog continues the 1.0.2 changelog, and there was also a 1.0.3 release continuing
+    the 1.0.2 changelog. The 1.0.3 changes are usually a subset of the 1.1.0 changes as
+    they are backported bugfixes.
+    '''
+    for section, changes in entry.changes.items():
+        if isinstance(changes, list):
+            entry.changes[section] = sorted(set(changes))
+    return entry
+
+
+def _add_rst_table_row(builder: RstBuilder, column_widths: t.List[int], row: t.List[str]):
+    lines = [[''] * len(column_widths)]
+    for i, value in enumerate(row):
+        for j, line in enumerate(value.splitlines()):
+            if j >= len(lines):
+                lines.append([''] * len(column_widths))
+            lines[j][i] = line
+    for line in lines:
+        parts = ['|']
+        for j, cell in enumerate(line):
+            parts.append(' ' + cell + ' ' * (1 + column_widths[j] - len(cell)) + '|')
+        builder.add_raw_rst(''.join(parts))
+
+
+def _add_rst_table_line(builder: RstBuilder, column_widths: t.List[int], sep: str):
+    parts = ['+']
+    for w in column_widths:
+        parts.append(sep * (w + 2) + '+')
+    builder.add_raw_rst(''.join(parts))
+
+
+def render_rst_table(builder: RstBuilder, headings: t.List[str],
+                     cells: t.List[t.List[str]]) -> None:
+    # Determine column widths
+    column_widths = []
+    for row in [headings] + cells:
+        while len(row) > len(column_widths):
+            column_widths.append(0)
+        for i, value in enumerate(row):
+            for line in value.splitlines():
+                column_widths[i] = max(column_widths[i], len(line))
+
+    # Add rows
+    _add_rst_table_line(builder, column_widths, '-')
+    _add_rst_table_row(builder, column_widths, headings)
+    _add_rst_table_line(builder, column_widths, '=')
+    for row in cells:
+        _add_rst_table_row(builder, column_widths, row)
+        _add_rst_table_line(builder, column_widths, '-')
+
+
 def append_changelog_changes_collections(builder: RstBuilder,
                                          collection_metadata: CollectionsMetadata,
                                          changelog_entry: ChangelogEntry,
@@ -37,20 +93,22 @@ def append_changelog_changes_collections(builder: RstBuilder,
 
     if changelog_entry.changed_collections:
         builder.add_section('Included Collections' if is_last else 'Changed Collections', 1)
+        builder.add_raw_rst(
+            'If not mentioned explicitly, the changes are reported in the combined changelog'
+            ' below.\n')
+        headings = [
+            'Collection',
+            f'Ansible {changelog_entry.prev_version}',
+            f'Ansible {changelog_entry.version_str}',
+            'Notes'
+        ]
+        cells = []
         for (
                 collector, collection_version, prev_collection_version
         ) in changelog_entry.changed_collections:
-            if is_last:
-                msg = f"{collector.collection} with version {collection_version}."
-                if prev_collection_version is not None:
-                    msg += f" This was upgraded from version {prev_collection_version}."
-            else:
-                if prev_collection_version is None:
-                    msg = f"{collector.collection} was upgraded to version {collection_version}."
-                else:
-                    msg = f"{collector.collection} was upgraded from"
-                    msg += f" version {prev_collection_version} to version {collection_version}."
-            msg += "\n"
+            row = [collector.collection, '', str(collection_version), '']
+            if prev_collection_version is not None:
+                row[1] = str(prev_collection_version)
             changelog = collector.changelog
             if changelog:
                 release_entries = changelog.generator.collect(
@@ -58,26 +116,29 @@ def append_changelog_changes_collections(builder: RstBuilder,
                     after_version=prev_collection_version,
                     until_version=collection_version)
                 if not release_entries:
-                    msg += "The collection did not have a changelog in this version."
+                    row[-1] = "The collection did not have a changelog in this version."
                 elif release_entries[0].empty:
-                    msg += "There are no changes recorded in the changelog."
+                    row[-1] = "There are no changes recorded in the changelog."
                 else:
                     result.append((
                         collector.collection,
                         f"{collector.collection}.",
                         changelog.generator,
-                        release_entries[0]))
-                    msg += "The changes are reported in the combined changelog below."
+                        optimize_release_entry(release_entries[0])))
             else:
                 metadata = collection_metadata.get_meta(collector.collection)
                 if metadata.changelog_url is not None:
-                    msg += "You can find the collection's changelog at"
-                    msg += f" `{metadata.changelog_url} <{metadata.changelog_url}>`_."
+                    row[-1] = (
+                        "You can find the collection's changelog at"
+                        f" `{metadata.changelog_url} <{metadata.changelog_url}>`_."
+                    )
                 else:
-                    msg += "Unfortunately, this collection does not provide changelog data in a"
-                    msg += " format that can be processed by the changelog generator."
-
-            builder.add_list_item(msg)
+                    row[-1] = (
+                        "Unfortunately, this collection does not provide changelog data in a"
+                        " format that can be processed by the changelog generator."
+                    )
+            cells.append(row)
+        render_rst_table(builder, headings, cells)
         builder.add_raw_rst('')
 
     return result
@@ -95,7 +156,7 @@ def append_changelog_changes_ansible(builder: RstBuilder,
     if not release_entries:
         return []
 
-    release_entry = release_entries[0]
+    release_entry = optimize_release_entry(release_entries[0])
 
     release_summary = release_entry.changes.pop('release_summary', None)
     if release_summary:
@@ -111,13 +172,19 @@ def append_changelog_changes_ansible(builder: RstBuilder,
 
 def append_changelog_changes_base(builder: RstBuilder,
                                   changelog_entry: ChangelogEntry) -> PluginDataT:
-    builder.add_section('Ansible-base', 1)
+    base_name = 'Ansible-base'
+    if (
+            changelog_entry.base_collector.latest.major > 2 or
+            changelog_entry.base_collector.latest.minor > 10
+    ):
+        base_name = 'Ansible-core'
+    builder.add_section(base_name, 1)
 
-    builder.add_raw_rst(f"Ansible {changelog_entry.version} contains Ansible-base "
+    builder.add_raw_rst(f"Ansible {changelog_entry.version} contains {base_name} "
                         f"version {changelog_entry.ansible_base_version}.")
     if changelog_entry.prev_ansible_base_version:
         if changelog_entry.prev_ansible_base_version == changelog_entry.ansible_base_version:
-            builder.add_raw_rst("This is the same version of Ansible-base as in "
+            builder.add_raw_rst(f"This is the same version of {base_name} as in "
                                 "the previous Ansible release.\n")
             return []
 
@@ -135,7 +202,7 @@ def append_changelog_changes_base(builder: RstBuilder,
         until_version=changelog_entry.ansible_base_version)
 
     if not release_entries:
-        builder.add_raw_rst("Ansible-base did not have a changelog in this version.")
+        builder.add_raw_rst(f"{base_name} did not have a changelog in this version.")
         return []
 
     release_entry = release_entries[0]
@@ -145,7 +212,7 @@ def append_changelog_changes_base(builder: RstBuilder,
         return []
 
     builder.add_raw_rst("The changes are reported in the combined changelog below.")
-    return [("Ansible-base", "ansible.builtin.", changelog.generator, release_entry)]
+    return [(base_name, "ansible.builtin.", changelog.generator, release_entry)]
 
 
 def common_start(a: t.List[t.Any], b: t.List[t.Any]) -> int:
@@ -163,9 +230,9 @@ def common_start(a: t.List[t.Any], b: t.List[t.Any]) -> int:
 PluginDumpT = t.List[t.Tuple[t.List[str], str, str]]
 
 
-def dump_plugins(builder: RstBuilder, plugins: PluginDumpT) -> None:
+def dump_items(builder: RstBuilder, items: PluginDumpT) -> None:
     last_title = []
-    for title, name, description in sorted(plugins):
+    for title, name, description in sorted(items):
         if title != last_title:
             if last_title:
                 builder.add_raw_rst('')
@@ -180,7 +247,7 @@ def dump_plugins(builder: RstBuilder, plugins: PluginDumpT) -> None:
 
 def add_plugins(builder: RstBuilder, data: PluginDataT) -> None:
     plugins: PluginDumpT = []
-    for name, prefix, _, release_entry in data:
+    for name, prefix, dummy, release_entry in data:
         if release_entry:
             for plugin_type, plugin_datas in release_entry.plugins.items():
                 for plugin_data in plugin_datas:
@@ -189,20 +256,38 @@ def add_plugins(builder: RstBuilder, data: PluginDataT) -> None:
                         ['New Plugins', plugin_type.title()],
                         prefix + plugin_data['name'],
                         plugin_data['description']))
-    dump_plugins(builder, plugins)
+    dump_items(builder, plugins)
+
+
+def add_objects(builder: RstBuilder, data: PluginDataT) -> None:
+    objects: PluginDumpT = []
+    for name, prefix, dummy, release_entry in data:
+        if release_entry:
+            for object_type, object_datas in release_entry.objects.items():
+                for object_data in object_datas:
+                    objects.append((
+                        ['New {type}s'.format(type=object_type.title())],
+                        prefix + object_data['name'],
+                        object_data['description']))
+    dump_items(builder, objects)
 
 
 def add_modules(builder: RstBuilder, data: PluginDataT) -> None:
     modules: PluginDumpT = []
-    for name, prefix, _, release_entry in data:
+    for name, prefix, dummy, release_entry in data:
         if release_entry:
             for module in release_entry.modules:
-                namespace = module['namespace'].split('.', 1) if module['namespace'] else []
+                namespace = module.get('namespace') or ''
+                if namespace.startswith('.ansible.collections.ansible_collections.'):
+                    # Work around old antsibull-changelog versions which suffer from
+                    # https://github.com/ansible-community/antsibull-changelog/issues/18
+                    namespace = ''
+                namespace = namespace.strip('.').split('.', 1) if namespace else []
                 modules.append((
                     ['New Modules', name] + [ns.replace('_', ' ').title() for ns in namespace],
                     prefix + module['name'],
                     module['description']))
-    dump_plugins(builder, modules)
+    dump_items(builder, modules)
 
 
 def create_title_adder(builder: RstBuilder, title: str,
@@ -266,7 +351,7 @@ def append_changelog(builder: RstBuilder,
     for section, section_title in DEFAULT_SECTIONS:
         maybe_add_section_title = create_title_adder(builder, section_title, 1)
 
-        for name, _, _, release_entry in data:
+        for name, dummy, dummy, release_entry in data:
             if not release_entry or release_entry.has_no_changes([section]):
                 continue
 
@@ -279,6 +364,7 @@ def append_changelog(builder: RstBuilder,
     # Adds new plugins and modules
     add_plugins(builder, data)
     add_modules(builder, data)
+    add_objects(builder, data)
 
     # Adds list of unchanged collections
     append_unchanged_collections(builder, changelog_entry)
@@ -309,7 +395,7 @@ def append_porting_guide_section(builder: RstBuilder, changelog_entry: Changelog
         next(maybe_add_section_title)
         if name:
             builder.add_section(name, 2)
-        entries[0].add_section_content(builder, section)
+        optimize_release_entry(entries[0]).add_section_content(builder, section)
         builder.add_raw_rst('')
 
     check_changelog(
@@ -317,8 +403,12 @@ def append_porting_guide_section(builder: RstBuilder, changelog_entry: Changelog
         changelog_entry.ansible_changelog,
         changelog_entry.version_str,
         str(changelog_entry.prev_version) if changelog_entry.prev_version else None)
+    is_core = (
+        changelog_entry.base_collector.latest.major > 2 or
+        changelog_entry.base_collector.latest.minor > 10
+    )
     check_changelog(
-        'Ansible-base',
+        'Ansible-core' if is_core else 'Ansible-base',
         changelog_entry.base_collector.changelog,
         changelog_entry.ansible_base_version,
         changelog_entry.prev_ansible_base_version)
@@ -336,7 +426,7 @@ def append_porting_guide(builder: RstBuilder, changelog_entry: ChangelogEntry) -
     maybe_add_title = create_title_adder(
         builder, 'Porting Guide for v{0}'.format(changelog_entry.version_str), 0)
 
-    for section in ['breaking_changes', 'major_changes']:
+    for section in ['known_issues', 'breaking_changes', 'major_changes']:
         append_porting_guide_section(builder, changelog_entry, maybe_add_title, section)
 
     if changelog_entry.removed_collections:
@@ -388,9 +478,11 @@ class ReleaseNotes:
     @staticmethod
     def _get_changelog_bytes(changelog: Changelog) -> bytes:
         builder = RstBuilder()
-        builder.set_title(
-            f"Ansible {changelog.ansible_version.major}.{changelog.ansible_version.minor}"
-            " Release Notes")
+        if changelog.ansible_version.major > 2:
+            version = f"{changelog.ansible_version.major}"
+        else:
+            version = f"{changelog.ansible_version.major}.{changelog.ansible_version.minor}"
+        builder.set_title(f"Ansible {version} Release Notes")
 
         if changelog.ansible_ancestor_version:
             builder.add_raw_rst(
@@ -400,65 +492,18 @@ class ReleaseNotes:
 
         builder.add_raw_rst('.. contents::\n  :local:\n  :depth: 2\n')
 
-        for index, changelog_entry in enumerate(changelog.entries):
+        entries = [entry for entry in changelog.entries if not entry.is_ancestor]
+        for index, changelog_entry in enumerate(entries):
             append_changelog(
                 builder,
                 changelog.collection_metadata,
                 changelog_entry,
-                is_last=index + 1 == len(changelog.entries))
+                is_last=index + 1 == len(entries))
 
         return builder.generate().encode('utf-8')
 
     @staticmethod
-    def _get_porting_guide_bytes(changelog: Changelog) -> bytes:
-        builder = RstBuilder()
-        builder.add_raw_rst(
-            f"..\n"
-            f"   THIS DOCUMENT IS AUTOMATICALLY GENERATED BY ANTSIBULL! PLEASE DO NOT EDIT"
-            f" MANUALLY! (YOU PROBABLY WANT TO EDIT porting_guide_base_"
-            f"{changelog.ansible_version.major}.{changelog.ansible_version.minor}.rst)\n")
-        builder.add_raw_rst(
-            f".. _porting_{changelog.ansible_version.major}."
-            f"{changelog.ansible_version.minor}_guide:\n")
-        builder.set_title(
-            f"Ansible {changelog.ansible_version.major}.{changelog.ansible_version.minor}"
-            f" Porting Guide")
-
-        builder.add_raw_rst(
-            # noqa: E501
-            ".. warning::"
-            "\n\n"
-            "        "
-            " In preparation for the release of 2.10, many plugins and modules have migrated to"
-            " Collections on  `Ansible Galaxy <https://galaxy.ansible.com>`_. For the current"
-            " development status of Collections and FAQ see `Ansible Collections Community Guide"
-            " <https://github.com/ansible-collections/overview/blob/master/README.rst>`_. We"
-            " expect the 2.10 Porting Guide to change frequently up to the 2.10 release. Follow"
-            " the conversations about collections on our various :ref:`communication` channels"
-            " for the latest information on the status of the ``devel`` branch."
-            "\n\n"
-            "This section discusses the behavioral changes between Ansible 2.9 and Ansible 2.10."
-            "\n\n"
-            "It is intended to assist in updating your playbooks, plugins and other parts of"
-            " your Ansible infrastructure so they will work with this version of Ansible."
-            "\n\n"
-            "We suggest you read this page along with the `Ansible Changelog for 2.10 <https://"
-            "github.com/ansible-community/ansible-build-data/blob/main/2.10/CHANGELOG-v2.10.rst>`_"
-            " to understand what updates you may need to make."
-            "\n\n"
-            "Since 2.10, ansible consists of two parts:\n"
-            "* ansible-base, which are the ansible command line tools with a very small selection"
-            " of plugins and modules, and\n"
-            "* a `set of collections <https://github.com/ansible-community/ansible-build-data/"
-            "blob/main/2.10/ansible.in>`_."
-            "\n\n"
-            "The :ref:`porting_2.10_guide_base` is included in this porting guide. The complete"
-            " list of porting guides can be found at :ref:`porting guides <porting_guides>`."
-            "\n"
-        )
-
-        builder.add_raw_rst('.. contents::\n  :local:\n  :depth: 2\n')
-
+    def _append_base_porting_guide_bytes(builder: RstBuilder, changelog: Changelog) -> None:
         base_porting_guide = changelog.base_collector.porting_guide
         if base_porting_guide:
             lines = base_porting_guide.decode('utf-8').splitlines()
@@ -478,18 +523,131 @@ class ReleaseNotes:
             if not found_empty:
                 print('WARNING: cannot find TOC of ansible-base porting guide!')
 
+    @staticmethod
+    def _get_porting_guide_bytes(changelog: Changelog) -> bytes:
+        if changelog.ansible_version.major > 2:
+            version = f"{changelog.ansible_version.major}"
+            base_version = changelog.base_collector.latest
+            base_version = f"{base_version.major}.{base_version.minor}"
+        else:
+            version = f"{changelog.ansible_version.major}.{changelog.ansible_version.minor}"
+            base_version = f"{changelog.ansible_version.major}.{changelog.ansible_version.minor}"
+        builder = RstBuilder()
+        builder.add_raw_rst(
+            f"..\n"
+            f"   THIS DOCUMENT IS AUTOMATICALLY GENERATED BY ANTSIBULL! PLEASE DO NOT EDIT"
+            f" MANUALLY! (YOU PROBABLY WANT TO EDIT porting_guide_base_{base_version}.rst)\n")
+        builder.add_raw_rst(f".. _porting_{version}_guide:\n")
+        builder.set_title(f"Ansible {version} Porting Guide")
+
+        if changelog.ansible_version.major == 2:
+            builder.add_raw_rst(
+                # noqa: E501
+                ".. warning::"
+                "\n\n"
+                "        "
+                " In Ansible 2.10, many plugins and modules have migrated to"
+                " Collections on `Ansible Galaxy <https://galaxy.ansible.com>`_. Your playbooks"
+                " should continue to work without any changes. We recommend you start using the"
+                " fully-qualified collection name (FQCN) in your playbooks as the explicit and"
+                " authoritative indicator of which collection to use as some collections may"
+                " contain duplicate module names."
+                " You can search the `index of all modules"
+                " <https://docs.ansible.com/ansible/2.10/collections/index_module.html>`_"
+                " to find the collection a module has been relocated to."
+                "\n\n"
+                "This section discusses the behavioral changes between Ansible 2.9 and"
+                " Ansible 2.10."
+                "\n\n"
+                "It is intended to assist in updating your playbooks, plugins and other parts of"
+                " your Ansible infrastructure so they will work with this version of Ansible."
+                "\n\n"
+                "We suggest you read this page along with the `Ansible 2.10 Changelog <https://"
+                "github.com/ansible-community/ansible-build-data/blob/main/2.10/"
+                "CHANGELOG-v2.10.rst>`_"
+                " to understand what updates you may need to make."
+                "\n\n"
+                "Since 2.10, Ansible consists of two parts:\n\n"
+                "* ansible-base, which includes the command line tools with a small selection"
+                " of plugins and modules, and\n"
+                "* a `set of collections <https://github.com/ansible-community/ansible-build-data/"
+                "blob/main/2.10/ansible.in>`_."
+                "\n\n"
+                "The :ref:`porting_2.10_guide_base` is included in this porting guide. The complete"
+                " list of porting guides can be found at :ref:`porting guides <porting_guides>`."
+                "\n"
+            )
+
+        builder.add_raw_rst('.. contents::\n  :local:\n  :depth: 2\n')
+
+        # Determine ansible-base/-core version in previous major release
+        prev_base_version = ''
+        if any(entry.is_ancestor for entry in changelog.entries):
+            # If there is an ancestor, the earliest ansible-base/-core version will be the
+            # version used in the previous major release.
+            prev_base_version = changelog.base_collector.earliest
+            prev_base_version = f"{prev_base_version.major}.{prev_base_version.minor}"
+
+        # Determine whether to include ansible-base/-core porting guide or not
+        if changelog.ansible_version.major == 2 or base_version != prev_base_version:
+            builder.add_raw_rst(
+                # noqa: E501
+                "\n"
+                f"We suggest you read this page along with the `Ansible {version} Changelog"
+                f" <https://github.com/ansible-community/ansible-build-data/blob/main/{version}/"
+                f"CHANGELOG-v{version}.rst>`_ to understand what updates you may need to make."
+                "\n")
+            ReleaseNotes._append_base_porting_guide_bytes(builder, changelog)
+        elif changelog.ansible_version.major == 3:
+            # Special message for Ansible 3
+            builder.add_raw_rst(
+                # noqa: E501
+                "\n"
+                "Ansible 3 is based on Ansible-Base 2.10, which is the same major release as"
+                " Ansible 2.10.  Therefore, there is no section on ansible-base in this porting"
+                " guide.  If you are upgrading from Ansible 2.9, please first consult the"
+                " Ansible 2.10 porting guide before continuing with the Ansible 3 porting guide."
+                "\n\n"
+                "We suggest you read this page along with the `Ansible 3 Changelog"
+                " <https://github.com/ansible-community/ansible-build-data/blob/main/3/"
+                "CHANGELOG-v3.rst>`_ to understand what updates you may need to make."
+                "\n"
+            )
+        else:
+            # Generic message if we again have two consecutive versions with the same ansible-core
+            prev_version = changelog.ansible_version.major - 1
+            prev_prev_version = changelog.ansible_version.major - 2
+            builder.add_raw_rst(
+                # noqa: E501
+                "\n"
+                f"Ansible {version} is based on Ansible-core {base_version}, which is the same"
+                f" major release as Ansible {prev_version}.  Therefore, there is no section on"
+                " ansible-core in this porting guide.  If you are upgrading from Ansible"
+                f" {prev_prev_version}, please first consult the Ansible {prev_version} porting"
+                f" guide before continuing with the Ansible {version} porting guide."
+                "\n\n"
+                f"We suggest you read this page along with the `Ansible {version} Changelog"
+                f" <https://github.com/ansible-community/ansible-build-data/blob/main/{version}/"
+                f"CHANGELOG-v{version}.rst>`_ to understand what updates you may need to make."
+                "\n"
+            )
+
         for porting_guide_entry in changelog.entries:
-            append_porting_guide(builder, porting_guide_entry)
+            if not porting_guide_entry.is_ancestor:
+                append_porting_guide(builder, porting_guide_entry)
 
         return builder.generate().encode('utf-8')
 
     @classmethod
     def build(cls, changelog: Changelog) -> 'ReleaseNotes':
+        if changelog.ansible_version.major > 2:
+            version = f"{changelog.ansible_version.major}"
+        else:
+            version = f"{changelog.ansible_version.major}.{changelog.ansible_version.minor}"
         return cls(
-            f"CHANGELOG-v{changelog.ansible_version.major}.{changelog.ansible_version.minor}.rst",
+            f"CHANGELOG-v{version}.rst",
             cls._get_changelog_bytes(changelog),
-            f"porting_guide_{changelog.ansible_version.major}"
-            f".{changelog.ansible_version.minor}.rst",
+            f"porting_guide_{version}.rst",
             cls._get_porting_guide_bytes(changelog),
         )
 
@@ -509,23 +667,32 @@ def build_changelog() -> int:
     app_ctx = app_context.app_ctx.get()
 
     ansible_version: PypiVer = app_ctx.extra['ansible_version']
-    deps_dir: str = app_ctx.extra['deps_dir']
-    dest_dir: str = app_ctx.extra['dest_dir']
+    data_dir: str = app_ctx.extra['data_dir']
+    dest_data_dir: str = app_ctx.extra['dest_data_dir']
     collection_cache: t.Optional[str] = app_ctx.extra['collection_cache']
 
-    changelog = get_changelog(ansible_version, deps_dir=deps_dir, collection_cache=collection_cache)
+    changelog = get_changelog(ansible_version, deps_dir=data_dir, collection_cache=collection_cache)
 
     release_notes = ReleaseNotes.build(changelog)
-    release_notes.write_changelog_to(dest_dir)
-    release_notes.write_porting_guide_to(dest_dir)
+    release_notes.write_changelog_to(dest_data_dir)
+    release_notes.write_porting_guide_to(dest_data_dir)
 
     missing_changelogs = []
-    for collector in changelog.collection_collectors:
+    last_entry = changelog.entries[0]
+    last_version_collectors = [
+        collector for collector in changelog.collection_collectors
+        if last_entry.version in last_entry.versions_per_collection[collector.collection]
+    ]
+    for collector in last_version_collectors:
         if collector.changelog is None:
             missing_changelogs.append(collector.collection)
     if missing_changelogs:
-        print(f"{len(missing_changelogs)} out of {len(changelog.collection_collectors)} collections"
+        print(f"{len(missing_changelogs)} out of {len(last_version_collectors)} collections"
               f" have no compatible changelog:")
-        for changelog in missing_changelogs:
-            print(f"    {changelog}")
+        for collection_name in missing_changelogs:
+            meta = changelog.collection_metadata.get_meta(collection_name)
+            entry = [collection_name]
+            if meta.changelog_url:
+                entry.append(f"(changelog URL: {meta.changelog_url})")
+            print(f"    {'  '.join(entry)}")
     return 0
