@@ -74,7 +74,8 @@ class ChangelogData:
         return cls(paths, config, ChangesData(config, '', changelog_data), flatmap=False)
 
     @classmethod
-    def ansible(cls, directory: t.Optional[str]) -> 'ChangelogData':
+    def ansible(cls, directory: t.Optional[str],
+                output_directory: t.Optional[str] = None) -> 'ChangelogData':
         paths = PathsConfig.force_ansible('')
 
         config = ChangelogConfig.default(paths, CollectionDetails(paths), 'Ansible')
@@ -86,7 +87,10 @@ class ChangelogData:
         changelog_path = ''
         if directory is not None:
             changelog_path = os.path.join(directory, 'changelog.yaml')
-        return cls(paths, config, ChangesData(config, changelog_path), flatmap=True)
+        changes = ChangesData(config, changelog_path)
+        if output_directory is not None:
+            changes.path = os.path.join(output_directory, 'changelog.yaml')
+        return cls(paths, config, changes, flatmap=True)
 
     @classmethod
     def concatenate(cls, changelogs: t.List['ChangelogData']) -> 'ChangelogData':
@@ -97,7 +101,8 @@ class ChangelogData:
             flatmap=changelogs[0].generator_flatmap)
 
     def add_ansible_release(self, version: str, date: datetime.date, release_summary: str) -> None:
-        add_release(self.config, self.changes, [], [], version, codename=None, date=date)
+        add_release(self.config, self.changes, [], [], version,
+                    codename=None, date=date, update_existing=True)
         release_date = self.changes.releases[version]
         if 'changes' not in release_date:
             release_date['changes'] = {}
@@ -126,7 +131,11 @@ def read_changelog_file(tarball_path: str, is_ansible_base=False) -> t.Optional[
 
 
 def get_porting_guide_filename(version: PypiVer):
-    return f"docs/docsite/rst/porting_guides/porting_guide_base_{version.major}.{version.minor}.rst"
+    if version.major == 2 and version.minor == 10:
+        base = 'porting_guide_base'
+    else:
+        base = 'porting_guide_core'
+    return f"docs/docsite/rst/porting_guides/{base}_{version.major}.{version.minor}.rst"
 
 
 def read_porting_guide_file(tarball_path: str, version: PypiVer) -> t.Optional[bytes]:
@@ -159,13 +168,14 @@ class CollectionChangelogCollector:
         changelog_data = yaml.load(changelog, Loader=yaml.SafeLoader)
         return ChangelogData.collection(self.collection, str(version), changelog_data)
 
-    async def download(self, collection_downloader: CollectionDownloader):
-        changelog = await self._get_changelog(self.latest, collection_downloader)
+    async def _download_changelog_stream(self, start_version: SemVer,
+                                         collection_downloader: CollectionDownloader
+                                         ) -> t.Optional[ChangelogData]:
+        changelog = await self._get_changelog(start_version, collection_downloader)
         if changelog is None:
-            return
-
-        changelog.changes.prune_versions(versions_after=None, versions_until=str(self.latest))
+            return None
 
+        changelog.changes.prune_versions(versions_after=None, versions_until=str(start_version))
         changelogs = [changelog]
         ancestor = changelog.changes.ancestor
         while ancestor is not None:
@@ -179,7 +189,33 @@ class CollectionChangelogCollector:
             changelogs.append(changelog)
             ancestor = changelog.changes.ancestor
 
-        self.changelog = ChangelogData.concatenate(changelogs)
+        return ChangelogData.concatenate(changelogs)
+
+    async def download(self, collection_downloader: CollectionDownloader):
+        missing_versions = set(self.versions)
+
+        while missing_versions:
+            missing_version = max(missing_versions)
+
+            # Try to get hold of changelog for this version
+            changelog = await self._download_changelog_stream(
+                missing_version, collection_downloader)
+            if changelog:
+                current_changelog = self.changelog
+                if current_changelog is None:
+                    # If we didn't have a changelog so far, start with it
+                    self.changelog = changelog
+                    missing_versions -= {SemVer(version) for version in changelog.changes.releases}
+                else:
+                    # Insert entries from changelog into combined changelog that are missing there
+                    for version, entry in changelog.changes.releases.items():
+                        sem_version = SemVer(version)
+                        if sem_version in missing_versions:
+                            current_changelog.changes.releases[version] = entry
+                            missing_versions.remove(sem_version)
+
+            # Make sure that this version isn't checked again
+            missing_versions -= {missing_version}
 
 
 class AnsibleBaseChangelogCollector:
@@ -198,38 +234,29 @@ class AnsibleBaseChangelogCollector:
         self.changelog = None
         self.porting_guide = None
 
-    async def _get_files(self, version: PypiVer,
-                         base_downloader: t.Callable[[str], t.Awaitable[str]]
-                         ) -> t.Tuple[t.Optional[ChangelogData], t.Optional[bytes]]:
+    async def _get_changelog_file(self, version: PypiVer,
+                                  base_downloader: t.Callable[[str], t.Awaitable[str]]
+                                  ) -> t.Optional[ChangelogData]:
         path = await base_downloader(str(version))
         if os.path.isdir(path):
-            pg_path, pg_filename = os.path.split(get_porting_guide_filename(version))
             changelog: t.Optional[ChangelogData] = None
-            porting_guide: t.Optional[bytes] = None
-            for root, _, files in os.walk(path):
+            for root, dummy, files in os.walk(path):
                 if 'changelog.yaml' in files:
                     with open(os.path.join(root, 'changelog.yaml'), 'rb') as f:
                         changelog = f.read()
                     changelog_data = yaml.load(changelog, Loader=yaml.SafeLoader)
                     changelog = ChangelogData.ansible_base(changelog_data)
-                if pg_filename in files:
-                    if os.path.join(path, pg_path) == root:
-                        with open(os.path.join(path, pg_path), 'rb') as f:
-                            porting_guide = f.read()
-            return changelog, porting_guide
+            return changelog
         if os.path.isfile(path) and path.endswith('.tar.gz'):
             changelog = read_changelog_file(path, is_ansible_base=True)
-            porting_guide = read_porting_guide_file(path, version)
             if changelog is None:
-                return None, porting_guide
+                return None
             changelog_data = yaml.load(changelog, Loader=yaml.SafeLoader)
-            return ChangelogData.ansible_base(changelog_data), porting_guide
-        return None, None
+            return ChangelogData.ansible_base(changelog_data)
+        return None
 
-    async def download(self, base_downloader: t.Callable[[str], t.Awaitable[str]]):
-        changelog, porting_guide = await self._get_files(self.latest, base_downloader)
-        if porting_guide:
-            self.porting_guide = porting_guide
+    async def download_changelog(self, base_downloader: t.Callable[[str], t.Awaitable[str]]):
+        changelog = await self._get_changelog_file(self.latest, base_downloader)
         if changelog is None:
             return
 
@@ -241,7 +268,7 @@ class AnsibleBaseChangelogCollector:
             ancestor_ver = PypiVer(ancestor)
             if ancestor_ver < self.earliest:
                 break
-            changelog, _ = await self._get_files(ancestor_ver, base_downloader)
+            changelog = await self._get_changelog_file(ancestor_ver, base_downloader)
             if changelog is None:
                 break
             changelog.changes.prune_versions(versions_after=None, versions_until=ancestor)
@@ -250,18 +277,9 @@ class AnsibleBaseChangelogCollector:
 
         self.changelog = ChangelogData.concatenate(changelogs)
 
-    async def download_github(self, aio_session: 'aiohttp.client.ClientSession'):
-        branch_url = (f"https://raw.githubusercontent.com/ansible/ansible/"
-                      f"stable-{self.latest.major}.{self.latest.minor}")
+    async def download_porting_guide(self, aio_session: 'aiohttp.client.ClientSession'):
+        branch_url = 'https://raw.githubusercontent.com/ansible/ansible/devel'
 
-        # Changelog
-        query_url = f"{branch_url}/changelogs/changelog.yaml"
-        async with aio_session.get(query_url) as response:
-            changelog = await response.read()
-        changelog_data = yaml.load(changelog, Loader=yaml.SafeLoader)
-        self.changelog = ChangelogData.ansible_base(changelog_data)
-
-        # Porting Guide
         query_url = f"{branch_url}/{get_porting_guide_filename(self.latest)}"
         async with aio_session.get(query_url) as response:
             self.porting_guide = await response.read()
@@ -283,18 +301,17 @@ async def collect_changelogs(collectors: t.List[CollectionChangelogCollector],
                 requestors = [
                     await pool.spawn(collector.download(downloader)) for collector in collectors
                 ]
-                if False:  # TODO: make this depend on version or something else...
-                    requestors.append(
-                        await pool.spawn(base_collector.download(base_downloader)))
-                else:
-                    requestors.append(
-                        await pool.spawn(base_collector.download_github(aio_session)))
+                requestors.append(
+                    await pool.spawn(base_collector.download_changelog(base_downloader)))
+                requestors.append(
+                    await pool.spawn(base_collector.download_porting_guide(aio_session)))
                 await asyncio.gather(*requestors)
 
 
 class ChangelogEntry:
     version: PypiVer
     version_str: str
+    is_ancestor: bool
 
     prev_version: t.Optional[PypiVer]
     base_versions: t.Dict[PypiVer, str]
@@ -315,7 +332,6 @@ class ChangelogEntry:
     def __init__(self, version: PypiVer, version_str: str,
                  prev_version: t.Optional[PypiVer],
                  ancestor_version: t.Optional[PypiVer],
-                 has_ancestor_versions: bool,
                  base_versions: t.Dict[PypiVer, str],
                  versions_per_collection: t.Dict[str, t.Dict[PypiVer, str]],
                  base_collector: AnsibleBaseChangelogCollector,
@@ -323,6 +339,7 @@ class ChangelogEntry:
                  collectors: t.List[CollectionChangelogCollector]):
         self.version = version
         self.version_str = version_str
+        self.is_ancestor = False if ancestor_version is None else ancestor_version == version
         self.prev_version = prev_version = prev_version or ancestor_version
         self.base_versions = base_versions
         self.versions_per_collection = versions_per_collection
@@ -353,7 +370,7 @@ class ChangelogEntry:
             )
             if prev_version:
                 if not prev_collection_version:
-                    if prev_version != ancestor_version or has_ancestor_versions:
+                    if prev_version != ancestor_version:
                         self.added_collections.append((collector, collection_version))
                 elif prev_collection_version == collection_version:
                     self.unchanged_collections.append((collector, collection_version))
@@ -459,9 +476,6 @@ def get_changelog(
         for deps in deps_data:
             dependencies[deps.ansible_version] = deps
 
-    # TODO: set to True if we have deps data for the ansible_ancestor_version
-    has_ancestor_versions = False
-
     base_versions: t.Dict[PypiVer, str] = dict()
     versions: t.Dict[str, t.Tuple[PypiVer, DependencyFileData]] = dict()
     versions_per_collection: t.Dict[str, t.Dict[PypiVer, str]] = defaultdict(dict)
@@ -482,7 +496,7 @@ def get_changelog(
     changelog = []
 
     sorted_versions = collect_versions(versions, ansible_changelog.config)
-    for index, (version_str, _) in enumerate(sorted_versions):
+    for index, (version_str, dummy) in enumerate(sorted_versions):
         version, deps = versions[version_str]
         prev_version = None
         if index + 1 < len(sorted_versions):
@@ -493,7 +507,6 @@ def get_changelog(
             version_str,
             prev_version,
             ansible_ancestor_version,
-            has_ancestor_versions,
             base_versions,
             versions_per_collection,
             base_collector,
