@@ -14,14 +14,14 @@ from packaging.version import Version as PypiVer
 
 from .. import app_context
 from ..app_logging import log
-from ..args import InvalidArgumentError, get_common_parser, normalize_common_options
-from ..config import load_config
-from ..new_ansible import new_ansible_command
+from ..args import InvalidArgumentError, get_toplevel_parser, normalize_toplevel_options
 from ..build_collection import build_collection_command
 from ..build_ansible_commands import (
     build_single_command, build_multiple_command, rebuild_single_command,
 )
 from ..build_changelog import build_changelog
+from ..config import load_config
+from ..new_ansible import new_ansible_command
 
 
 mlog = log.fields(mod=__name__)
@@ -44,9 +44,21 @@ ARGS_MAP = {'new-ansible': new_ansible_command,
 
 
 def _normalize_build_options(args: argparse.Namespace) -> None:
-    args.dest_dir = os.path.expanduser(os.path.expandvars(args.dest_dir))
-    if not os.path.isdir(args.dest_dir):
-        raise InvalidArgumentError(f'{args.dest_dir} must be an existing directory')
+    if not os.path.isdir(args.data_dir):
+        raise InvalidArgumentError(f'{args.data_dir} must be an existing directory')
+
+
+def _normalize_build_write_data_options(args: argparse.Namespace) -> None:
+    if args.command not in (
+            'new-ansible', 'single', 'rebuild-single', 'multiple', 'changelog',
+            'new-acd', 'build-single', 'build-multiple'):
+        return
+
+    if args.dest_data_dir is None:
+        args.dest_data_dir = args.data_dir
+
+    if not os.path.isdir(args.dest_data_dir):
+        raise InvalidArgumentError(f'{args.dest_data_dir} must be an existing directory')
 
 
 def _normalize_new_release_options(args: argparse.Namespace) -> None:
@@ -60,18 +72,24 @@ def _normalize_new_release_options(args: argparse.Namespace) -> None:
         return
 
     if args.pieces_file is None:
-        args.pieces_file = os.path.join(args.dest_dir, DEFAULT_PIECES_FILE)
+        args.pieces_file = DEFAULT_PIECES_FILE
 
-    if not os.path.isfile(args.pieces_file):
-        raise InvalidArgumentError(f'The pieces file, {args.pieces_file}, must already'
+    pieces_path = os.path.join(args.data_dir, args.pieces_file)
+    if not os.path.isfile(pieces_path):
+        raise InvalidArgumentError(f'The pieces file, {pieces_path}, must already'
                                    ' exist. It should contain one namespace.collection'
                                    ' per line')
 
     if args.build_file is None:
         basename = os.path.basename(os.path.splitext(args.pieces_file)[0])
-        args.build_file = (
-            f'{basename}-{args.ansible_version.major}.{args.ansible_version.minor}.build'
-        )
+        if args.ansible_version.major > 2:
+            args.build_file = (
+                f'{basename}-{args.ansible_version.major}.build'
+            )
+        else:
+            args.build_file = (
+                f'{basename}-{args.ansible_version.major}.{args.ansible_version.minor}.build'
+            )
 
 
 def _normalize_release_build_options(args: argparse.Namespace) -> None:
@@ -88,29 +106,38 @@ def _normalize_release_build_options(args: argparse.Namespace) -> None:
     if args.command not in ('single', 'multiple', 'rebuild-single'):
         return
 
-    if args.build_file is None:
-        args.build_file = (DEFAULT_FILE_BASE
-                           + f'-{args.ansible_version.major}.{args.ansible_version.minor}.build')
+    compat_version_part = (
+        f'{args.ansible_version.major}' if args.ansible_version.major > 2
+        else f'{args.ansible_version.major}.{args.ansible_version.minor}'
+    )
 
-    if not os.path.isfile(args.build_file):
-        raise InvalidArgumentError(f'The build file, {args.build_file} must already exist.'
+    if args.build_file is None:
+        args.build_file = DEFAULT_FILE_BASE + f'-{compat_version_part}.build'
+
+    build_filename = os.path.join(args.data_dir, args.build_file)
+    if not os.path.isfile(build_filename):
+        raise InvalidArgumentError(f'The build file, {build_filename} must already exist.'
                                    ' It should contains one namespace.collection and range'
                                    ' of versions per line')
 
     if args.deps_file is None:
-        major_minor = f'-{args.ansible_version.major}.{args.ansible_version.minor}'
+        version_suffix = f'-{compat_version_part}'
         basename = os.path.basename(os.path.splitext(args.build_file)[0])
-        if basename.endswith(major_minor):
-            basename = basename[:-len(major_minor)]
+        if basename.endswith(version_suffix):
+            basename = basename[:-len(version_suffix)]
 
         args.deps_file = f'{basename}-{args.ansible_version}.deps'
 
+    if args.command in ('single', 'multiple'):
+        if not os.path.isdir(args.sdist_dir):
+            raise InvalidArgumentError(f'{args.sdist_dir} must be an existing directory')
+
 
 def _normalize_release_rebuild_options(args: argparse.Namespace) -> None:
     if args.command not in ('rebuild-single', ):
         return
 
-    deps_filename = os.path.join(args.dest_dir, args.deps_file)
+    deps_filename = os.path.join(args.data_dir, args.deps_file)
     if not os.path.isfile(deps_filename):
         raise InvalidArgumentError(f'The dependency file, {deps_filename} must already exist.')
 
@@ -128,6 +155,9 @@ def _normalize_collection_build_options(args: argparse.Namespace) -> None:
     if args.deps_file is None:
         args.deps_file = DEFAULT_FILE_BASE + f'{args.ansible_version}.deps'
 
+    if not os.path.isdir(args.collection_dir):
+        raise InvalidArgumentError(f'{args.collection_dir} must be an existing directory')
+
 
 def parse_args(program_name: str, args: List[str]) -> argparse.Namespace:
     """
@@ -138,13 +168,17 @@ def parse_args(program_name: str, args: List[str]) -> argparse.Namespace:
     :returns: A :python:`argparse.Namespace`
     :raises InvalidArgumentError: Whenever there's something wrong with the arguments.
     """
-    common_parser = get_common_parser()
-
-    build_parser = argparse.ArgumentParser(add_help=False, parents=[common_parser])
+    build_parser = argparse.ArgumentParser(add_help=False)
     build_parser.add_argument('ansible_version', type=PypiVer,
                               help='The X.Y.Z version of Ansible that this will be for')
-    build_parser.add_argument('--dest-dir', default='.',
-                              help='Directory to write the output to')
+    build_parser.add_argument('--data-dir', default='.',
+                              help='Directory to read .build and .deps files from')
+
+    build_write_data_parser = argparse.ArgumentParser(add_help=False, parents=[build_parser])
+    build_write_data_parser.add_argument('--dest-data-dir', default=None,
+                                         help='Directory to write .build and .deps files to,'
+                                         ' as well as changelog and porting guide if applicable.'
+                                         '  Defaults to --data-dir')
 
     cache_parser = argparse.ArgumentParser(add_help=False)
     cache_parser.add_argument('--collection-cache', default=None,
@@ -156,55 +190,70 @@ def parse_args(program_name: str, args: List[str]) -> argparse.Namespace:
     build_step_parser = argparse.ArgumentParser(add_help=False)
     build_step_parser.add_argument('--build-file', default=None,
                                    help='File containing the list of collections with version'
-                                   ' ranges. The default is to look for'
-                                   ' $DEFAULT_FILE_BASE-X.Y.build inside of --dest-dir')
+                                   ' ranges.  This is considered to be relative to'
+                                   ' --build-data-dir.  The default is'
+                                   ' $DEFAULT_FILE_BASE-X.Y.build')
     build_step_parser.add_argument('--deps-file', default=None,
                                    help='File which will be written containing the list of'
                                    ' collections at versions which were included in this version'
-                                   ' of Ansible. The default is to place'
-                                   ' $BASENAME_OF_BUILD_FILE-X.Y.Z.deps into --dest-dir')
+                                   ' of Ansible.  This is considered to be relative to'
+                                   ' --build-data-dir.  The default is'
+                                   ' $BASENAME_OF_BUILD_FILE-X.Y.Z.deps')
+
+    feature_freeze_parser = argparse.ArgumentParser(add_help=False)
+    feature_freeze_parser.add_argument('--feature-frozen', action='store_true',
+                                       help='If this is given, then do not allow collections whose'
+                                       ' version implies there are new features.')
+
+    parser = get_toplevel_parser(prog=program_name,
+                                 description='Script to manage building Ansible')
 
-    parser = argparse.ArgumentParser(prog=program_name,
-                                     description='Script to manage building Ansible')
     subparsers = parser.add_subparsers(title='Subcommands', dest='command',
                                        help='for help use antsibull-build SUBCOMMANDS -h')
     subparsers.required = True
 
-    new_parser = subparsers.add_parser('new-ansible', parents=[build_parser],
+    new_parser = subparsers.add_parser('new-ansible', parents=[build_write_data_parser],
                                        description='Generate a new build description from the'
                                        ' latest available versions of ansible-base and the'
                                        ' included collections')
     new_parser.add_argument('--pieces-file', default=None,
-                            help='File containing a list of collections to include.  The'
-                            f' default is to look for {DEFAULT_PIECES_FILE} inside of --dest-dir')
+                            help='File containing a list of collections to include.  This is'
+                            ' considered to be relative to --data-dir.  The default is'
+                            f' {DEFAULT_PIECES_FILE}')
     new_parser.add_argument('--build-file', default=None,
                             help='File which will be written which contains the list'
-                            ' of collections with version ranges.  The default is to'
-                            ' place $BASENAME_OF_PIECES_FILE-X.Y.build into --dest-dir')
+                            ' of collections with version ranges.  This is considered to be'
+                            ' relative to --dest-data-dir.  The default is'
+                            ' $BASENAME_OF_PIECES_FILE-X.Y.build')
 
     build_single_parser = subparsers.add_parser('single',
-                                                parents=[build_parser, cache_parser,
-                                                         build_step_parser],
+                                                parents=[build_write_data_parser, cache_parser,
+                                                         build_step_parser, feature_freeze_parser],
                                                 description='Build a single-file Ansible')
-
+    build_single_parser.add_argument('--sdist-dir', default='.',
+                                     help='Directory to write the generated sdist tarball to')
     build_single_parser.add_argument('--debian', action='store_true',
                                      help='Include Debian/Ubuntu packaging files in'
                                      ' the resulting output directory')
 
     rebuild_single_parser = subparsers.add_parser('rebuild-single',
-                                                  parents=[build_parser, cache_parser,
+                                                  parents=[build_write_data_parser, cache_parser,
                                                            build_step_parser],
                                                   description='Rebuild a single-file Ansible from'
                                                               ' a dependency file')
-
+    rebuild_single_parser.add_argument('--sdist-dir', default='.',
+                                       help='Directory to write the generated sdist tarball to')
     rebuild_single_parser.add_argument('--debian', action='store_true',
                                        help='Include Debian/Ubuntu packaging files in'
                                        ' the resulting output directory')
 
     build_multiple_parser = subparsers.add_parser('multiple',
-                                                  parents=[build_parser, cache_parser,
-                                                           build_step_parser],
+                                                  parents=[build_write_data_parser, cache_parser,
+                                                           build_step_parser,
+                                                           feature_freeze_parser],
                                                   description='Build a multi-file Ansible')
+    build_multiple_parser.add_argument('--sdist-dir', default='.',
+                                       help='Directory to write the generated sdist tarballs to')
 
     collection_parser = subparsers.add_parser('collection',
                                               parents=[build_parser],
@@ -212,17 +261,17 @@ def parse_args(program_name: str, args: List[str]) -> argparse.Namespace:
                                               ' install Ansible')
     collection_parser.add_argument('--deps-file', default=None,
                                    help='File which contains the list of collections and'
-                                   ' versions which were included in this version of Ansible'
-                                   f' The default is to look for {DEFAULT_FILE_BASE}-X.Y.Z.deps'
-                                   ' inside of --dest-dir')
+                                   ' versions which were included in this version of Ansible.'
+                                   '  This is considered to be relative to --data-dir.'
+                                   f'  The default is {DEFAULT_FILE_BASE}-X.Y.Z.deps')
+    build_parser.add_argument('--collection-dir', default='.',
+                              help='Directory to write collection to')
 
-    changelog_parser = subparsers.add_parser('changelog',
-                                             parents=[build_parser, cache_parser],
-                                             description='Build the Ansible changelog')
-    changelog_parser.add_argument('--deps-dir', required=True,
-                                  help='Directory which contains the versioning data')
+    subparsers.add_parser('changelog',
+                          parents=[build_write_data_parser, cache_parser],
+                          description='Build the Ansible changelog')
 
-    # Backwards compat.
+    # Backwards compat
     subparsers.add_parser('new-acd', add_help=False, parents=[new_parser])
     subparsers.add_parser('build-single', add_help=False, parents=[build_single_parser])
     subparsers.add_parser('build-multiple', add_help=False, parents=[build_multiple_parser])
@@ -231,8 +280,9 @@ def parse_args(program_name: str, args: List[str]) -> argparse.Namespace:
     args: argparse.Namespace = parser.parse_args(args)
 
     # Validation and coercion
-    normalize_common_options(args)
+    normalize_toplevel_options(args)
     _normalize_build_options(args)
+    _normalize_build_write_data_options(args)
     _normalize_new_release_options(args)
     _normalize_release_build_options(args)
     _normalize_release_rebuild_options(args)
@@ -287,8 +337,4 @@ def main() -> int:
         :3: version in an input file does not match with the version specified on the command line
         :4: Needs to be run on a newer version of Python
     """
-    if sys.version_info < (3, 8):
-        print('Needs Python 3.8 or later')
-        return 4
-
     return run(sys.argv)
