@@ -21,13 +21,28 @@ from ...augment_docs import augment_docs
 from ...collections import install_together
 from ...compat import asyncio_run, best_get_loop
 from ...dependency_files import DepsFile
-from ...docs_parsing.ansible_doc import get_ansible_plugin_info
+from ...extra_docs import load_collections_extra_docs
+from ...docs_parsing.parsing import get_ansible_plugin_info
 from ...docs_parsing.fqcn import get_fqcn_parts
+from ...docs_parsing.routing import (
+    find_stubs,
+    load_all_collection_routing,
+    remove_redirect_duplicates,
+)
 from ...galaxy import CollectionDownloader
 from ...logging import log
 from ...schemas.docs import DOCS_SCHEMAS
 from ...venv import VenvRunner, FakeVenvRunner
-from ...write_docs import output_all_plugin_rst, output_collection_index, output_indexes
+from ...write_docs import (
+    output_all_plugin_rst,
+    output_all_plugin_stub_rst,
+    output_collection_index,
+    output_collection_namespace_indexes,
+    output_indexes,
+    output_plugin_indexes,
+    output_extra_docs,
+)
+from ...utils.transformations import get_collection_namespaces
 
 if t.TYPE_CHECKING:
     import semantic_version as semver
@@ -48,7 +63,7 @@ async def retrieve(ansible_base_version: str,
     """
     Download ansible-base and the collections.
 
-    :arg ansible_base_version: Version of ansible-base to download.
+    :arg ansible_base_version: Version of ansible-base/-core to download.
     :arg collections: Map of collection names to collection versions to download.
     :arg tmp_dir: The directory to download into
     :arg galaxy_server: URL to the galaxy server.
@@ -182,35 +197,57 @@ async def normalize_all_plugin_info(plugin_info: t.Mapping[str, t.Mapping[str, t
     return new_plugin_info, nonfatal_errors
 
 
-def get_collection_contents(plugin_info: t.Mapping[str, t.Mapping[str, t.Any]],
-                            nonfatal_errors: PluginErrorsRT
-                            ) -> t.DefaultDict[str, t.DefaultDict[str, t.Dict[str, str]]]:
+def get_plugin_contents(plugin_info: t.Mapping[str, t.Mapping[str, t.Any]],
+                        nonfatal_errors: PluginErrorsRT
+                        ) -> t.DefaultDict[str, t.DefaultDict[str, t.Dict[str, str]]]:
     """
-    Return the plugins which are in each collection.
+    Return the collections with their plugins for every plugin type.
 
     :arg plugin_info: Mapping of plugin type to a mapping of plugin name to plugin record.
         The plugin_type, plugin_name, and short_description from plugin_records are used.
     :arg nonfatal_errors: mapping of plugin type to plugin name to list of error messages.
         The plugin_type and plugin_name are used.
+    :returns: A Mapping of plugin type to a mapping of collection name to a mapping of plugin names
+        to short_descriptions.
+    plugin_type:
+        collection:
+            - plugin_short_name: short_description
+    """
+    plugin_contents = defaultdict(lambda: defaultdict(dict))
+    # Some plugins won't have an entry in the plugin_info because documentation failed to parse.
+    # Those should be documented in the nonfatal_errors information.
+    for plugin_type, plugin_list in nonfatal_errors.items():
+        for plugin_name, dummy_ in plugin_list.items():
+            namespace, collection, short_name = get_fqcn_parts(plugin_name)
+            plugin_contents[plugin_type]['.'.join((namespace, collection))][short_name] = ''
+
+    for plugin_type, plugin_list in plugin_info.items():
+        for plugin_name, plugin_desc in plugin_list.items():
+            namespace, collection, short_name = get_fqcn_parts(plugin_name)
+            plugin_contents[plugin_type]['.'.join((namespace, collection))][short_name] = (
+                plugin_desc['doc']['short_description'])
+
+    return plugin_contents
+
+
+def get_collection_contents(plugin_content: t.Mapping[str, t.Mapping[str, t.Mapping[str, str]]],
+                            ) -> t.DefaultDict[str, t.Dict[str, t.Mapping[str, str]]]:
+    """
+    Return the plugins which are in each collection.
+
+    :arg plugin_content: Mapping of plugin type to a mapping of collection name to a mapping of
+        plugin name to short description.
     :returns: A Mapping of collection name to a mapping of plugin type to a mapping of plugin names
         to short_descriptions.
     collection:
         plugin_type:
             - plugin_short_name: short_description
     """
-    collection_plugins = defaultdict(lambda: defaultdict(dict))
-    # Some plugins won't have an entry in the plugin_info because documentation failed to parse.
-    # Those should be documented in the nonfatal_errors information.
-    for plugin_type, plugin_list in nonfatal_errors.items():
-        for plugin_name, dummy_ in plugin_list.items():
-            namespace, collection, short_name = get_fqcn_parts(plugin_name)
-            collection_plugins['.'.join((namespace, collection))][plugin_type][short_name] = ''
+    collection_plugins = defaultdict(dict)
 
-    for plugin_type, plugin_list in plugin_info.items():
-        for plugin_name, plugin_desc in plugin_list.items():
-            namespace, collection, short_name = get_fqcn_parts(plugin_name)
-            collection_plugins['.'.join((namespace, collection))][plugin_type][short_name] = (
-                plugin_desc['doc']['short_description'])
+    for plugin_type, collection_data in plugin_content.items():
+        for collection_name, plugin_data in collection_data.items():
+            collection_plugins[collection_name][plugin_type] = plugin_data
 
     return collection_plugins
 
@@ -220,6 +257,7 @@ def generate_docs_for_all_collections(venv: t.Union[VenvRunner, FakeVenvRunner],
                                       dest_dir: str,
                                       flog,
                                       collection_names: t.Optional[t.List[str]] = None,
+                                      create_indexes: bool = True,
                                       squash_hierarchy: bool = False) -> None:
     """
     Create documentation for a set of installed collections.
@@ -232,16 +270,25 @@ def generate_docs_for_all_collections(venv: t.Union[VenvRunner, FakeVenvRunner],
     :arg flog: A logger instance.
     :arg collection_names: Optional list of collection names. If specified, only documentation
                            for these collections will be collected and generated.
+    :arg create_indexes: Whether to create the collection index and plugin indexes. By default,
+                         they are created.
     :arg squash_hierarchy: If set to ``True``, no directory hierarchy will be used.
                            Undefined behavior if documentation for multiple collections are
                            created.
     """
 
     # Get the info from the plugins
-    plugin_info = asyncio_run(get_ansible_plugin_info(
+    plugin_info, collection_metadata = asyncio_run(get_ansible_plugin_info(
         venv, collection_dir, collection_names=collection_names))
-    flog.notice('Finished parsing info from plugins')
+    flog.notice('Finished parsing info from plugins and collections')
     # flog.fields(plugin_info=plugin_info).debug('Plugin data')
+    # flog.fields(
+    #     collection_metadata=collection_metadata).debug('Collection metadata')
+
+    # Load collection routing information
+    collection_routing = asyncio_run(load_all_collection_routing(collection_metadata))
+    flog.notice('Finished loading collection routing information')
+    # flog.fields(collection_routing=collection_routing).debug('Collection routing infos')
 
     """
     # Turn these into some sort of decorator that will choose to dump or load the values
@@ -255,6 +302,10 @@ def generate_docs_for_all_collections(venv: t.Union[VenvRunner, FakeVenvRunner],
         plugin_info = json.load(f)
     """
 
+    remove_redirect_duplicates(plugin_info, collection_routing)
+    stubs_info = find_stubs(plugin_info, collection_routing)
+    # flog.fields(stubs_info=stubs_info).debug('Stubs info')
+
     plugin_info, nonfatal_errors = asyncio_run(normalize_all_plugin_info(plugin_info))
     flog.fields(errors=len(nonfatal_errors)).notice('Finished data validation')
     augment_docs(plugin_info)
@@ -280,22 +331,51 @@ def generate_docs_for_all_collections(venv: t.Union[VenvRunner, FakeVenvRunner],
     flog.debug('Finished loading errors')
     """
 
-    collection_info = get_collection_contents(plugin_info, nonfatal_errors)
+    # Load collection extra docs data
+    extra_docs_data = asyncio_run(load_collections_extra_docs(
+        {name: data.path for name, data in collection_metadata.items()}))
+    flog.debug('Finished getting collection extra docs data')
+
+    plugin_contents = get_plugin_contents(plugin_info, nonfatal_errors)
+    collection_to_plugin_info = get_collection_contents(plugin_contents)
+    # Make sure collections without documentable plugins are mentioned
+    for collection in collection_metadata:
+        collection_to_plugin_info[collection]
     flog.debug('Finished getting collection data')
 
-    # Only build top-level index if no collection names were specified
-    if collection_names is None:
-        asyncio_run(output_collection_index(collection_info, dest_dir))
-        flog.notice('Finished writing collection index')
+    collection_namespaces = get_collection_namespaces(collection_to_plugin_info.keys())
 
-    asyncio_run(output_indexes(collection_info, dest_dir, squash_hierarchy=squash_hierarchy))
+    # Only build top-level index if requested
+    if create_indexes:
+        asyncio_run(output_collection_index(
+            collection_to_plugin_info, collection_namespaces, dest_dir))
+        flog.notice('Finished writing collection index')
+        asyncio_run(output_collection_namespace_indexes(collection_namespaces, dest_dir))
+        flog.notice('Finished writing collection namespace index')
+        asyncio_run(output_plugin_indexes(plugin_contents, dest_dir))
+        flog.notice('Finished writing plugin indexes')
+
+    asyncio_run(output_indexes(collection_to_plugin_info, dest_dir,
+                               collection_metadata=collection_metadata,
+                               squash_hierarchy=squash_hierarchy,
+                               extra_docs_data=extra_docs_data))
     flog.notice('Finished writing indexes')
 
-    asyncio_run(output_all_plugin_rst(collection_info, plugin_info,
+    asyncio_run(output_all_plugin_stub_rst(stubs_info, dest_dir,
+                                           collection_metadata=collection_metadata,
+                                           squash_hierarchy=squash_hierarchy))
+    flog.debug('Finished writing plugin stubs')
+
+    asyncio_run(output_all_plugin_rst(collection_to_plugin_info, plugin_info,
                                       nonfatal_errors, dest_dir,
+                                      collection_metadata=collection_metadata,
                                       squash_hierarchy=squash_hierarchy))
     flog.debug('Finished writing plugin docs')
 
+    asyncio_run(output_extra_docs(dest_dir, extra_docs_data,
+                                  squash_hierarchy=squash_hierarchy))
+    flog.debug('Finished writing extra extra docs docs')
+
 
 def generate_docs() -> int:
     """
