@@ -21,6 +21,10 @@ mlog = log.fields(mod=__name__)
 #: Valid choices for a logging level field
 LEVEL_CHOICES_F = p.Field(..., regex='^(CRITICAL|ERROR|WARNING|NOTICE|INFO|DEBUG|DISABLED)$')
 
+#: Valid choices for a logging level field
+DOC_PARSING_BACKEND_CHOICES_F = p.Field(
+    'ansible-internal', regex='^(ansible-doc|ansible-internal)$')
+
 #: Valid choice of the logging version field
 VERSION_CHOICES_F = p.Field(..., regex=r'1\.0')
 
@@ -131,6 +135,7 @@ class ConfigModel(BaseModel):
     pypi_url: p.HttpUrl = 'https://pypi.org/'
     thread_max: int = 80
     max_retries: int = 10
+    doc_parsing_backend: str = DOC_PARSING_BACKEND_CHOICES_F
 
     @p.validator('process_max', pre=True)
     def convert_to_none(cls, value):
@@ -152,7 +157,7 @@ def find_config_files(conf_files: t.Iterable[str]) -> t.List[str]:
     flog = mlog.fields(func='find_config_file')
     flog.fields(conf_files=conf_files).debug('Enter')
 
-    paths = [os.path.abspath(os.path.expanduser(os.path.expandvars(p))) for p in conf_files]
+    paths = [os.path.abspath(p) for p in conf_files]
     flog.fields(paths=paths).info('Paths to check')
 
     config_files = []
@@ -175,7 +180,7 @@ def read_config(filename: str) -> ConfigModel:
     flog = mlog.fields(func='read_config')
     flog.debug('Enter')
 
-    filename = os.path.abspath(os.path.expanduser(os.path.expandvars(filename)))
+    filename = os.path.abspath(filename)
 
     flog.fields(filename=filename).info('loading config file')
     raw_config_data = perky.load(filename)
@@ -208,7 +213,8 @@ def load_config(conf_files: t.Union[t.Iterable[str], str, None] = None) -> t.Dic
     elif conf_files is None:
         conf_files = ()
 
-    available_files = find_config_files(itertools.chain((SYSTEM_CONFIG_FILE, USER_CONFIG_FILE),
+    user_config_file = os.path.expanduser(USER_CONFIG_FILE)
+    available_files = find_config_files(itertools.chain((SYSTEM_CONFIG_FILE, user_config_file),
                                                         conf_files))
 
     includes = list(available_files)
