@@ -49,29 +49,31 @@ def _parse_name_version_spec_file(filename: str) -> DependencyFileData:
     ansible_base_version: Optional[str] = None
     ansible_version: Optional[str] = None
 
-    with open(filename, 'r') as f:
-        for line in f:
-            record = [entry.strip() for entry in line.split(':', 1)]
+    for line in parse_pieces_file(filename):
+        record = [entry.strip() for entry in line.split(':', 1)]
 
-            if record[0] in ('_ansible_version', '_acd_version'):
-                if ansible_version is not None:
-                    raise InvalidFileFormat(f'{filename} specified _ansible_version/_acd_version'
-                                            ' more than once')
-                ansible_version = record[1]
-                continue
+        if record[0] in ('_ansible_version', '_acd_version'):
+            if ansible_version is not None:
+                raise InvalidFileFormat(f'{filename} specified _ansible_version/_acd_version'
+                                        ' more than once')
+            ansible_version = record[1]
+            continue
 
-            if record[0] == '_ansible_base_version':
-                if ansible_base_version is not None:
-                    raise InvalidFileFormat(f'{filename} specified _ansible_base_version'
-                                            ' more' ' than once')
-                ansible_base_version = record[1]
-                continue
+        if record[0] == '_ansible_base_version':
+            if ansible_base_version is not None:
+                raise InvalidFileFormat(f'{filename} specified _ansible_base_version'
+                                        ' more' ' than once')
+            ansible_base_version = record[1]
+            continue
 
-            deps[record[0]] = record[1]
+        deps[record[0]] = record[1]
 
-    if ansible_base_version is None or ansible_version is None:
+    if ansible_base_version is None:
         raise InvalidFileFormat(f'{filename} was invalid.  It did not contain'
-                                ' required fields')
+                                ' the required ansible_base_version field')
+    if ansible_version is None:
+        raise InvalidFileFormat(f'{filename} was invalid.  It did not contain'
+                                ' the required ansible_version field')
 
     return DependencyFileData(ansible_version, ansible_base_version, deps)
 
@@ -142,19 +144,26 @@ class BuildFile:
         Write a build dependency file.
 
         A build dependency file records the collections that went into a given Ansible release along
-        with the exact version of the collection that was included.
+        with the range of versions that are allowed for that collection.  The range is meant to
+        define the set of collection versions that are compatible with what was present in the
+        collection as of the first beta release, when we feature freeze the collections.
 
         :arg ansible_version: The version of Ansible that is being recorded.
         :arg ansible_base_version: The version of Ansible base that will be depended on.
-        :arg dependencies: Dictionary with keys of collection names and values of versions.
+        :arg dependencies: Dictionary with keys of collection names and values of the latest
+            versions of those collections.
         """
         records = []
         for dep, version in dependencies.items():
-            records.append(f'{dep}: >={version.major}.0.0,<{version.next_major()}')
+            records.append(f'{dep}: >={version.major}.{version.minor}.0,<{version.next_major()}')
         records.sort()
 
         with open(self.filename, 'w') as f:
-            f.write(f'_ansible_version: {ansible_version.major}.{ansible_version.minor}\n')
+            if ansible_version.major > 2:
+                # Ansible 3.0.0 and newer use semver, so we only need the major version
+                f.write(f'_ansible_version: {ansible_version.major}\n')
+            else:
+                f.write(f'_ansible_version: {ansible_version.major}.{ansible_version.minor}\n')
             f.write(f'_ansible_base_version: {ansible_base_version}\n')
             f.write('\n'.join(records))
             f.write('\n')
