@@ -5,8 +5,8 @@
 
 import asyncio
 import json
-import os
 import sys
+import os
 import traceback
 import typing as t
 from concurrent.futures import ThreadPoolExecutor
@@ -19,6 +19,7 @@ from ..constants import DOCUMENTABLE_PLUGINS
 from ..logging import log
 from ..vendored.json_utils import _filter_non_json_lines
 from .fqcn import get_fqcn_parts
+from . import _get_environment, ParsingError, AnsibleCollectionMetadata
 
 if t.TYPE_CHECKING:
     from ..venv import VenvRunner, FakeVenvRunner
@@ -26,45 +27,6 @@ if t.TYPE_CHECKING:
 
 mlog = log.fields(mod=__name__)
 
-#: Clear Ansible environment variables that set paths where plugins could be found.
-ANSIBLE_PATH_ENVIRON: t.Dict[str, str] = os.environ.copy()
-ANSIBLE_PATH_ENVIRON.update({'ANSIBLE_COLLECTIONS_PATH': '/dev/null',
-                             'ANSIBLE_ACTION_PLUGINS': '/dev/null',
-                             'ANSIBLE_CACHE_PLUGINS': '/dev/null',
-                             'ANSIBLE_CALLBACK_PLUGINS': '/dev/null',
-                             'ANSIBLE_CLICONF_PLUGINS': '/dev/null',
-                             'ANSIBLE_CONNECTION_PLUGINS': '/dev/null',
-                             'ANSIBLE_FILTER_PLUGINS': '/dev/null',
-                             'ANSIBLE_HTTPAPI_PLUGINS': '/dev/null',
-                             'ANSIBLE_INVENTORY_PLUGINS': '/dev/null',
-                             'ANSIBLE_LOOKUP_PLUGINS': '/dev/null',
-                             'ANSIBLE_LIBRARY': '/dev/null',
-                             'ANSIBLE_MODULE_UTILS': '/dev/null',
-                             'ANSIBLE_NETCONF_PLUGINS': '/dev/null',
-                             'ANSIBLE_ROLES_PATH': '/dev/null',
-                             'ANSIBLE_STRATEGY_PLUGINS': '/dev/null',
-                             'ANSIBLE_TERMINAL_PLUGINS': '/dev/null',
-                             'ANSIBLE_TEST_PLUGINS': '/dev/null',
-                             'ANSIBLE_VARS_PLUGINS': '/dev/null',
-                             'ANSIBLE_DOC_FRAGMENT_PLUGINS': '/dev/null',
-                             })
-try:
-    del ANSIBLE_PATH_ENVIRON['PYTHONPATH']
-except KeyError:
-    # We just wanted to make sure there was no PYTHONPATH set...
-    # all python libs will come from the venv
-    pass
-try:
-    del ANSIBLE_PATH_ENVIRON['ANSIBLE_COLLECTIONS_PATHS']
-except KeyError:
-    # ANSIBLE_COLLECTIONS_PATHS is the deprecated name replaced by
-    # ANSIBLE_COLLECTIONS_PATH
-    pass
-
-
-class ParsingError(Exception):
-    """Error raised while parsing plugins for documentation."""
-
 
 def _process_plugin_results(plugin_type: str,
                             plugin_names: t.Iterable[str],
@@ -197,27 +159,56 @@ async def _get_plugin_info(plugin_type: str, ansible_doc: 'sh.Command',
     return results
 
 
-def _get_environment(collection_dir: t.Optional[str]) -> t.Dict[str, str]:
-    env = ANSIBLE_PATH_ENVIRON.copy()
-    if collection_dir is not None:
-        env['ANSIBLE_COLLECTIONS_PATH'] = collection_dir
-    else:
-        # Copy ANSIBLE_COLLECTIONS_PATH and ANSIBLE_COLLECTIONS_PATHS from the
-        # original environment.
-        for env_var in ('ANSIBLE_COLLECTIONS_PATH', 'ANSIBLE_COLLECTIONS_PATHS'):
-            try:
-                del env[env_var]
-            except KeyError:
-                pass
-            if env_var in os.environ:
-                env[env_var] = os.environ[env_var]
-    return env
+def get_collection_metadata(venv: t.Union['VenvRunner', 'FakeVenvRunner'],
+                            env: t.Dict[str, str],
+                            collection_names: t.Optional[t.List[str]] = None,
+                            ) -> t.Dict[str, AnsibleCollectionMetadata]:
+    collection_metadata = {}
+
+    # Obtain ansible.builtin version
+    if collection_names is None or 'ansible.builtin' in collection_names:
+        venv_ansible = venv.get_command('ansible')
+        ansible_version_cmd = venv_ansible('--version', _env=env)
+        raw_result = ansible_version_cmd.stdout.decode('utf-8', errors='surrogateescape')
+        path: t.Optional[str] = None
+        version: t.Optional[str] = None
+        for line in raw_result.splitlines():
+            if line.strip().startswith('ansible python module location'):
+                path = line.split('=', 2)[1].strip()
+            if line.startswith('ansible '):
+                version = line[len('ansible '):]
+        collection_metadata['ansible.builtin'] = AnsibleCollectionMetadata(
+            path=path, version=version)
+
+    # Obtain collection versions
+    venv_ansible_galaxy = venv.get_command('ansible-galaxy')
+    ansible_collection_list_cmd = venv_ansible_galaxy('collection', 'list', _env=env)
+    raw_result = ansible_collection_list_cmd.stdout.decode('utf-8', errors='surrogateescape')
+    current_base_path = None
+    for line in raw_result.splitlines():
+        parts = line.split()
+        if len(parts) >= 2:
+            if parts[0] == '#':
+                current_base_path = parts[1]
+            else:
+                collection_name = parts[0]
+                version = parts[1]
+                if '.' in collection_name:
+                    if collection_names is None or collection_name in collection_names:
+                        namespace, name = collection_name.split('.', 2)
+                        collection_metadata[collection_name] = AnsibleCollectionMetadata(
+                            path=os.path.join(current_base_path, namespace, name),
+                            version=None if version == '*' else version)
+
+    return collection_metadata
 
 
 async def get_ansible_plugin_info(venv: t.Union['VenvRunner', 'FakeVenvRunner'],
                                   collection_dir: t.Optional[str],
                                   collection_names: t.Optional[t.List[str]] = None
-                                  ) -> t.Dict[str, t.Dict[str, t.Any]]:
+                                  ) -> t.Tuple[
+                                      t.Mapping[str, t.Mapping[str, t.Any]],
+                                      t.Mapping[str, AnsibleCollectionMetadata]]:
     """
     Retrieve information about all of the Ansible Plugins.
 
@@ -227,13 +218,18 @@ async def get_ansible_plugin_info(venv: t.Union['VenvRunner', 'FakeVenvRunner'],
                          search path for Ansible.
     :arg collection_names: Optional list of collections. If specified, will only collect
                            information for plugins in these collections.
-    :returns: A nested directory structure that looks like::
+    :returns: An tuple. The first component is a nested directory structure that looks like:
 
-        plugin_type:
-            plugin_name:  # Includes namespace and collection.
-                {information from ansible-doc --json.  See the ansible-doc documentation for more
-                 info.}
+            plugin_type:
+                plugin_name:  # Includes namespace and collection.
+                    {information from ansible-doc --json.  See the ansible-doc documentation
+                     for more info.}
+
+        The second component is a Mapping of collection names to metadata.
     """
+    flog = mlog.fields(func='get_ansible_plugin_info')
+    flog.debug('Enter')
+
     env = _get_environment(collection_dir)
 
     # Setup an sh.Command to run ansible-doc from the venv with only the collections we
@@ -299,4 +295,8 @@ async def get_ansible_plugin_info(venv: t.Union['VenvRunner', 'FakeVenvRunner'],
         # done so, we want to then fail by raising one of the exceptions.
         raise ParsingError('Parsing of plugins failed')
 
-    return plugin_map
+    flog.debug('Retrieving collection metadata')
+    collection_metadata = get_collection_metadata(venv, env, collection_names)
+
+    flog.debug('Leave')
+    return (plugin_map, collection_metadata)
