@@ -72,7 +72,9 @@ class GalaxyClient:
         :arg version_url: url to the page to retrieve.
         :returns: List of the all the versions of the collection.
         """
-        async with retry_get(self.aio_session, versions_url, params=self.params,
+        params = self.params.copy()
+        params['page_size'] = '100'
+        async with retry_get(self.aio_session, versions_url, params=params,
                              acceptable_error_codes=[404]) as response:
             if response.status == 404:
                 raise NoSuchCollection(f'No collection found at: {versions_url}')
@@ -154,12 +156,15 @@ class GalaxyClient:
         return collection_info
 
     async def get_latest_matching_version(self, collection: str,
-                                          version_spec: str) -> semver.Version:
+                                          version_spec: str,
+                                          pre: bool = False) -> semver.Version:
         """
         Get the latest version of a collection that matches a specification.
 
         :arg collection: Namespace.collection identifying a collection.
         :arg version_spec: String specifying the allowable versions.
+        :kwarg pre: If True, allow prereleases (versions which have the form X.Y.Z.SOMETHING).
+            This is **not** for excluding 0.Y.Z versions.  The default is False.
         :returns: :obj:`semantic_version.Version` of the latest collection version that satisfied
             the specification.
 
@@ -172,6 +177,9 @@ class GalaxyClient:
 
         spec = semver.SimpleSpec(version_spec)
         for version in (v for v in versions if v in spec):
+            # If we're excluding prereleases and this is a prerelease, then skip it.
+            if not pre and version.prerelease:
+                continue
             return version
 
         # No matching versions were found
