@@ -9,6 +9,10 @@ from html import escape as html_escape
 
 from jinja2.runtime import Undefined
 
+from ..logging import log
+
+
+mlog = log.fields(mod=__name__)
 
 # Warning: If you add to this, then you also have to change ansible-doc
 # (ansible/cli/__init__.py) in the ansible/ansible repository
@@ -25,17 +29,24 @@ _RULER = re.compile(r"\bHORIZONTALLINE\b")
 def html_ify(text):
     ''' convert symbols like I(this is in italics) to valid HTML '''
 
-    text = html_escape(text)
-    text = _ITALIC.sub(r"<em>\1</em>", text)
-    text = _BOLD.sub(r"<b>\1</b>", text)
-    text = _MODULE.sub(r"<span class='module'>\1</span>", text)
-    text = _URL.sub(r"<a href='\1'>\1</a>", text)
-    text = _REF.sub(r"<span class='module'>\1</span>", text)
-    text = _LINK.sub(r"<a href='\2'>\1</a>", text)
-    text = _CONST.sub(r"<code>\1</code>", text)
-    text = _RULER.sub(r"<hr/>", text)
+    flog = mlog.fields(func='html_ify')
+    flog.fields(text=text).debug('Enter')
+    _counts = {}
 
-    return text.strip()
+    text = html_escape(text)
+    text, _counts['italic'] = _ITALIC.subn(r"<em>\1</em>", text)
+    text, _counts['bold'] = _BOLD.subn(r"<b>\1</b>", text)
+    text, _counts['module'] = _MODULE.subn(r"<span class='module'>\1</span>", text)
+    text, _counts['url'] = _URL.subn(r"<a href='\1'>\1</a>", text)
+    text, _counts['ref'] = _REF.subn(r"<span class='module'>\1</span>", text)
+    text, _counts['link'] = _LINK.subn(r"<a href='\2'>\1</a>", text)
+    text, _counts['const'] = _CONST.subn(r"<code>\1</code>", text)
+    text, _counts['ruler'] = _RULER.subn(r"<hr/>", text)
+
+    text = text.strip()
+    flog.fields(counts=_counts).info('Number of macros converted to html equivalents')
+    flog.debug('Leave')
+    return text
 
 
 def documented_type(text):
@@ -62,15 +73,21 @@ def do_max(seq):
 def rst_ify(text):
     ''' convert symbols like I(this is in italics) to valid restructured text '''
 
-    text = _ITALIC.sub(r"*\1*", text)
-    text = _BOLD.sub(r"**\1**", text)
-    text = _MODULE.sub(r":ref:`\1 <ansible_collections.\1_module>`", text)
-    text = _LINK.sub(r"`\1 <\2>`_", text)
-    text = _URL.sub(r"\1", text)
-    text = _REF.sub(r":ref:`\1 <\2>`", text)
-    text = _CONST.sub(r"``\1``", text)
-    text = _RULER.sub(f"\n\n{'-' * 13}\n\n", text)
+    flog = mlog.fields(func='html_ify')
+    flog.fields(text=text).debug('Enter')
+    _counts = {}
 
+    text, _counts['italic'] = _ITALIC.subn(r"*\1*", text)
+    text, _counts['bold'] = _BOLD.subn(r"**\1**", text)
+    text, _counts['module'] = _MODULE.subn(r":ref:`\1 <ansible_collections.\1_module>`", text)
+    text, _counts['url'] = _LINK.subn(r"`\1 <\2>`_", text)
+    text, _counts['ref'] = _URL.subn(r"\1", text)
+    text, _counts['link'] = _REF.subn(r":ref:`\1 <\2>`", text)
+    text, _counts['const'] = _CONST.subn(r"``\1``", text)
+    text, _counts['ruler'] = _RULER.subn(f"\n\n{'-' * 13}\n\n", text)
+
+    flog.fields(counts=_counts).info('Number of macros converted to rst equivalents')
+    flog.debug('Leave')
     return text
 
 
