@@ -134,7 +134,7 @@ REQUIRED_ENV_VAR_F = p.Field(..., regex='[A-Z_]+')
 
 #: option types are a set of strings that represent the types handled by argspec.
 OPTION_TYPE_F = p.Field('str', regex='^(bits|bool|bytes|dict|float|int|json|jsonarg|list'
-                        '|path|raw|sid|str)$')
+                        '|path|raw|sid|str|tmppath|pathspec|pathlist)$')
 
 #: Constrained string type for version numbers
 REQUIRED_VERSION_F = p.Field(..., regex='^([0-9][0-9.]+)$')
@@ -152,7 +152,8 @@ COLLECTION_NAME_F = p.Field(default='', regex='^([^.]+\\.[^.]+)$')
 REQUIRED_COLLECTION_NAME_OR_EMPTY_STR_F = p.Field('', regex='^([^.]+\\.[^.]+)?$')
 
 #: Constrained string listing the possible types of a return field
-RETURN_TYPE_F = p.Field('str', regex='^(bool|complex|dict|float|int|list|str)$')
+RETURN_TYPE_F = p.Field('str', regex='^(any|bits|bool|bytes|complex|dict|float|int|json'
+                        '|jsonarg|list|path|sid|str|pathspec|pathlist)$')
 
 
 def is_json_value(value: t.Any) -> bool:
@@ -232,6 +233,19 @@ def normalize_option_type_names(obj):
     if obj == 'lists':
         return 'list'
 
+    if obj in ('tmp', 'temppath'):
+        return 'tmppath'
+
+    return obj
+
+
+def normalize_return_type_names(obj):
+    """Normalize common mispellings of return type names."""
+    obj = normalize_option_type_names(obj)
+
+    if obj == 'raw':
+        return 'any'
+
     return obj
 
 
